; ************************************************************************************************
; * DMX zu tpm2-Umsetzung fr SEDU-Board	mit M644p										 	 *
; *																								 *
; *	empfngt bis zu 512 Kanle ber DMX und gibt sie ber USB als tpm2 aus					 	 *
; *	mit Bootloader-Support																 		 *
; *																								 *
; *	by Pesi	- Version 1.0 - 01.09.2013															 *
; ************************************************************************************************

.include "m644Pdef.inc"							; verwendeter C: Mega644p

.include "Definitions.inc"						; Definitionen Register, Konstanten etc.

.CSEG

.include "Interrupts.inc"						; IRQ-Vektor-Tabelle

; ================================================================================================
; Subroutinen
; ------------------------------------------------------------------------------------------------

.include "ISR_Byte_Received_USB.inc"			; Empfangsroutine USB-Port (fr Bootloader-Check)

.include "ISR_Byte_Received_DMX.inc"			; Empfangsroutine DMX ber RS485-Port

.include "Timer0OVF.inc"						; Timerroutine fr PWM, Zeitbasis, etc.

; ================================================================================================

_reset:

.include "Configuration.inc"					; Ports setzen, Timer konfigurieren etc.

; ================================================================================================
; Hauptschleife, schickt die in einer ISR ber DMX empfangenen Daten als tpm2 ber USB raus
; ------------------------------------------------------------------------------------------------

_main:

		sbrs	Flags,FL_DMX_FR					; Flag fr Frame empfangen gesetzt?
		rjmp	_main							; nein, nix machen
		cbr		Flags, (1<<FL_DMX_FR)			; ja, Flag fr Frame empfangen lschen

_usb_send_wait_0:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_wait_0

		ldi		temp0, 0xC9						; tpm2-Startcode schicken
		sts		UDR0, temp0

_usb_send_wait_1:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_wait_1

		ldi		temp0, 0xDA						; Kennung fr Datenframe schicken
		sts		UDR0, temp0

_usb_send_wait_2:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_wait_2

		ldi		temp0, High(DMX_CHANNELS)		; Highbyte Framegre schicken
		sts		UDR0, temp0

_usb_send_wait_3:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_wait_3

		ldi		temp0, Low(DMX_CHANNELS)		; Lowbyte Framegre schicken
		sts		UDR0, temp0

		ldi		XL, Low(DMX_CHANNELS)			; Gre Datenblock
		ldi		XH, High(DMX_CHANNELS)			; Gre Datenblock

		ldi		ZL, Low(DMX_In_Buffer)			; Pointer auf DMX-Buffer
		ldi		ZH, High(DMX_In_Buffer)

_usb_send_loop:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_loop

		ld		temp0, Z+						; Byte aus Puffer holen
		sts		UDR0, temp0						; verschicken

		sbiw	XH:XL, 1						; verschickte Bytes zhlen
		brne	_usb_send_loop					; noch nicht alle geschickt, nchstes

_usb_send_wait_5:

		lds		temp0, UCSR0A					; warten auf Tx bereit
		sbrs	temp0, UDRE0
		rjmp	_usb_send_wait_5

		ldi		temp0, 0x36						; tpm2-Blockende-Byte schicken
		sts		UDR0, temp0

		rjmp	_main							; von vorne

; ================================================================================================
; Daten im Flash und EEPROM
; ------------------------------------------------------------------------------------------------

Bootstring:

.db		"seduboot"								; Startcode fr Bootloader

; ================================================================================================
